/*
 * common functions for (parallel) cellular automaton
 *
 * (c) 2016 Steffen Christgau
 *
 * configuration initialization based on
 * (c) 1996,1997 Peter Sanders, Ingo Boesnach
 *
 */
#include <stdint.h>
#include <stdlib.h>
#include <stdio.h>
#include <assert.h>

#include "ca_common.h"

void ca_init(int argc, char** argv, int *lines, int *its)
{
	assert(argc == 3);

	*lines = atoi(argv[1]);
	*its = atoi(argv[2]);

	assert(*lines > 0);
}

/* random starting configuration */
void ca_init_config(line_t *buf, int lines, int skip_lines)
{
	volatile int scratch;

	srand(424243);

	/* let the RNG spin for some rounds (used for distributed initialization) */
	for (int y = 1;  y <= skip_lines;  y++) {
		for (int x = 1;  x <= XSIZE;  x++) {
			scratch = scratch + (rand() % 2);
		}
	}

	for (int y = 1;  y <= lines;  y++) {
		for (int x = 1;  x <= XSIZE;  x++) {
			buf[y][x] = rand() % 2;
		}
	}
}

static void ca_clean_ghost_zones(line_t *buf, int lines)
{
	for (int y = 0; y < lines; y++) {
		buf[y][0] = 0;
		buf[y][XSIZE + 1] = 0;
	}
}

void ca_report(line_t *buf, int lines)
{
	ca_clean_ghost_zones(buf, lines);
	fwrite(buf, sizeof(*buf), lines, stdout);
}
